"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const url_1 = require("url");
const get_path_info_1 = require("./get-path-info");
const normalize_config_1 = require("./normalize-config");
const read_1 = require("./read");
const watcher_1 = require("./watcher");
/**
 * A CodeEngine plugin that reads files from the filesystem.
 */
function filesystem(options) {
    let config = normalize_config_1.normalizeConfig(options);
    let path;
    let source;
    let watcher;
    return {
        name: "Filesystem Source",
        filter: config.filter,
        /**
         * One-time initialization
         */
        async initialize() {
            // Resolve the full source path based on CodeEngine's CWD
            path = await get_path_info_1.getPathInfo(this.engine, config);
            // The file:// URL of the source path
            source = url_1.pathToFileURL(path.absolutePath).href;
        },
        /**
         * Read file(s) from the filesystem
         */
        async read(run) {
            if ("filename" in path) {
                return read_1.readFile(path, config, run);
            }
            else {
                return read_1.readDir(path, config, run);
            }
        },
        /**
         * Re-read files that were marked as changed by other plugins.
         */
        async processFile(file, run) {
            // If the file has no contents, and it has changed, and it's within our source path,
            // then re-read its contents from disk
            if (file.size === 0 && file.change && file.source.startsWith(source)) {
                file.contents = await config.fs.promises.readFile(file.source);
            }
            return file;
        },
        /**
         * Watches the filesystem and yields any changes that are detected.
         */
        async watch() {
            watcher = watcher || new watcher_1.Watcher(this.engine, path, config);
            return watcher.iterable;
        },
        /**
         * Stops watching the filesystem for changes.
         */
        async dispose() {
            if (watcher) {
                let promise = watcher.dispose();
                watcher = undefined;
                await promise;
            }
        }
    };
}
exports.filesystem = filesystem;
//# sourceMappingURL=plugin.js.map